using Polly;

var builder = WebApplication.CreateBuilder(args);

// Add services to the container.

builder.Services.AddHttpClient("InventoryApi", client =>
{
    client.BaseAddress = new Uri("http://localhost:5000/");
    client.DefaultRequestHeaders.Add("Accept", "application/json");
}).AddPolicyHandler(Policy.HandleResult<HttpResponseMessage>
 (r => !r.IsSuccessStatusCode).RetryAsync(3));

var app = builder.Build();

int _requestCount = 0; 

app.MapGet("/Inventory/{id}", (int id) =>
{
    _requestCount++;

    if (_requestCount % 4 == 0) // only one of out four requests will succeed
    {
        return Results.Ok(15);
    }

    System.Console.WriteLine($"{_requestCount} Something went wrong");
    return Results.Problem("Something went wrong");
});

app.MapGet("/Catalog/{id}", async (int id, IHttpClientFactory httpClientFactory) =>
{
    var httpClient = httpClientFactory.CreateClient("InventoryApi");

    string requestEndpoint = $"Inventory/{id}";

    HttpResponseMessage response = await httpClient.GetAsync(requestEndpoint);

    if (response.IsSuccessStatusCode)
    {
        int itemsInStock = await response.Content.ReadFromJsonAsync<int>();
        return Results.Ok(itemsInStock);
    }
    return Results.Problem("Something went wrong");

});

app.Run();